@@ -1,4 +1,4 @@
-/* $Id: guppi-slinreg-statviewer.h,v 1.4 2000/10/27 18:22:50 trow Exp $ */
+/* $Id: guppi-slinreg-statviewer.h,v 1.5 2001/01/16 23:36:05 trow Exp $ */
 
 /*
  * guppi-slinreg-statviewer.h
@@ -27,7 +27,7 @@
 #ifndef _INC_GUPPI_SLINREG_STATVIEWER_H
 #define _INC_GUPPI_SLINREG_STATVIEWER_H
 
-#include <gnome.h>
+/* #include <gnome.h> */
 #include <guppi-simple-linreg.h>
 #include <guppi-defs.h>
 
@@ -39,4 +39,4 @@ END_GUPPI_DECLS
 
 #endif /* _INC_GUPPI_SLINREG-STATVIEWER_H */
 
-/* $Id: guppi-slinreg-statviewer.h,v 1.4 2000/10/27 18:22:50 trow Exp $ */
+/* $Id: guppi-slinreg-statviewer.h,v 1.5 2001/01/16 23:36:05 trow Exp $ */
