@@ -1,4 +1,4 @@
-/* $Id: guppi.c,v 1.53 2001/01/16 23:36:10 trow Exp $ */
+/* $Id: guppi.c,v 1.56 2001/05/06 08:26:33 trow Exp $ */
 /*
  * guppi.c
  *
@@ -37,13 +37,13 @@
 #include <guppi-data-init.h>
 #include <guppi-data-ui-init.h>
 #include <guppi-stat-init.h>
+#include <guppi-eval-init.h>
 #include <guppi-plot-init.h>
 #include <guppi-plot-plug-in.h>
 #ifdef USING_BONOBO
 #  include <guppi-bonobo.h>
 #  include <liboaf/liboaf.h>
 #endif
-#include "app_scm.h"
 #include "term.h"
 #include "file-open.h"
 #include "plug-ins.h"
@@ -100,7 +100,7 @@ guppi_add_menu (const gchar * path, GnomeUIInfo * menuinfo)
 }
 
 
-/****************************************************************************/
+/**************************************************************************/
 
 static void
 load_rc_files (void)
@@ -121,9 +121,9 @@ load_rc_files (void)
   }
 }
 
-/****************************************************************************/
+/**************************************************************************/
 
-/******  Signal handlers  ******/
+/*****  Signal handlers  *****/
 
 /*
   On the first try, we attempt a graceful exit.
@@ -167,7 +167,7 @@ setup_signal_handlers (void)
   sigaction (SIGHUP, &ash, NULL);
 }
 
-/****************************************************************************/
+/**************************************************************************/
 
 static gboolean batch_mode = FALSE;
 static gboolean dump_paths = FALSE;
@@ -203,12 +203,12 @@ real_main (void *closure, int argc, char *argv[])
   guppi_splash_message (_("Initializing"), _("Data UI Library"));
   guppi_data_ui_init ();
 
+  guppi_splash_message (_("Initializing"), _("Eval Library"));
+  guppi_eval_init ();
+
   guppi_splash_message (_("Initializing"), _("Plot Library"));
   guppi_plot_init ();
 
-  guppi_splash_message (_("Initializing"), _("GuppiAppGuile"));
-  guppi_app_scm_init ();
-
   if (!batch_mode) {
     guppi_splash_message (_("Initializing"), _("Main Guppi Window"));
     guppi_app = GNOME_APP (guppi_app_new ());
@@ -422,4 +422,4 @@ main (int argc, char *argv[])
 }
 
 
-/* $Id: guppi.c,v 1.53 2001/01/16 23:36:10 trow Exp $ */
+/* $Id: guppi.c,v 1.56 2001/05/06 08:26:33 trow Exp $ */
