@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-seq-string.h,v 1.6 2000/10/27 18:22:59 trow Exp $ */
+/* $Id: guppi-seq-string.h,v 1.7 2001/01/16 23:36:22 trow Exp $ */
 
 /*
  * guppi-seq-string.h
@@ -28,7 +28,7 @@
 #ifndef _INC_GUPPI_SEQ_STRING_H
 #define _INC_GUPPI_SEQ_STRING_H
 
-#include <gtk/gtk.h>
+/* #include <gtk/gtk.h> */
 
 #include "guppi-seq.h"
 
@@ -88,4 +88,4 @@ END_GUPPI_DECLS
 
 #endif /* _INC_GUPPI_SEQ_STRING_H */
 
-/* $Id: guppi-seq-string.h,v 1.6 2000/10/27 18:22:59 trow Exp $ */
+/* $Id: guppi-seq-string.h,v 1.7 2001/01/16 23:36:22 trow Exp $ */
