@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-layout.c,v 1.22 2001/01/16 23:36:26 trow Exp $ */
+/* $Id: guppi-layout.c,v 1.24 2001/05/06 08:26:43 trow Exp $ */
 
 /*
  * guppi-layout.c
@@ -33,228 +33,39 @@
 #include <guppi-useful.h>
 #include "guppi-layout.h"
 
-#define EPSILON 1e-8
-
 static GtkObjectClass *parent_class = NULL;
 
-/* An ultra-minimal matrix type, for notational convenience only. */
-
-typedef struct _Matrix Matrix;
-struct _Matrix {
-  gint r, c;
-  double *data;
+struct CustomLUInfo {
+  double x0, y0, x1, y1;
 };
 
-#define matrix_rows(m) ((m)->r)
-#define matrix_cols(m) ((m)->c)
-
-static Matrix *
-matrix_new (gint r, gint c)
-{
-  Matrix *m;
-  g_return_val_if_fail (r > 0 && c > 0, NULL);
-  m = guppi_new (Matrix, 1);
-  m->r = r;
-  m->c = c;
-  m->data = guppi_new0 (double, r * c);
-  return m;
-}
-
-#ifdef EXTRA_UNUSED_FN
-static Matrix *
-matrix_copy (Matrix * m)
-{
-  Matrix *copy = matrix_new (matrix_rows (m), matrix_cols (m));
-  memcpy (copy->data, m->data,
-	  matrix_rows (m) * matrix_cols (m) * sizeof (double));
-  return copy;
-}
-#endif
-
-static void
-matrix_free (Matrix * m)
-{
-  if (m == NULL)
-    return;
-
-  guppi_free (m->data);
-  m->data = NULL;
-  m->r = 0;
-  m->c = 0;
-  guppi_free (m);
-}
-
-
-#define mat_ptr(m,i,j) ((m)->data + ((i)*((m)->c)) +(j))
-#define mat_col_incr(m, p) ((p)+1)
-#define mat_row_incr(m, p) ((p)+((m)->c))
-#define mat(m,i,j) (*(mat_ptr(m,i,j)))
-
-#if 0
-static void
-matrix_print (Matrix * m)
-{
-  gint i, j;
-  for (i = 0; i < matrix_rows (m); ++i) {
-    for (j = 0; j < matrix_cols (m); ++j)
-      g_print ("%9.2f ", mat (m, i, j));
-    g_print ("\n");
-  }
-}
-#endif
-
-static void
-matrix_normalize_row (Matrix * m, gint r)
-{
-  double sumsq = 0, norm;
-  double *p;
-  double *q;
-  gint i;
-
-  p = q = mat_ptr (m, r, 0);
-  for (i = 0; i < matrix_cols (m); ++i) {
-    sumsq += (*q) * (*q);
-    q = mat_col_incr (m, q);
-  }
-
-  norm = sqrt (sumsq);
-
-  for (i = 0; i < matrix_cols (m); ++i) {
-    *p /= norm;
-    p = mat_col_incr (m, p);
-  }
-}
-
 static gboolean
-matrix_row_is_nonzero (Matrix * m, gint r)
+custom_solve_fallback (GuppiMatrix *m, GuppiVector *vec, gint i, gpointer user_data)
 {
-  double *p = mat_ptr (m, r, 0);
-  gint i;
-  for (i = 0; i < matrix_cols (m); ++i) {
-    if (fabs (*p) > EPSILON)
-      return TRUE;
-    p = mat_col_incr (m, p);
+  struct CustomLUInfo *info = user_data;
+
+  switch (i % 4) {
+  case 0: return info->x0;
+  case 1: return info->x1;
+  case 2: return info->y1;
+  case 3: return info->y0;
   }
-  return FALSE;
+
+  return TRUE;
 }
 
-static gboolean
-matrix_column_is_nonzero (Matrix * m, gint c)
+static GuppiVector *
+custom_solve (GuppiMatrix *m, GuppiVector *vec,
+	      double x0, double x1, double y0, double y1)
 {
-  double *p = mat_ptr (m, 0, c);
-  gint i;
-  for (i = 0; i < matrix_rows (m); ++i) {
-    if (fabs (*p) > EPSILON)
-      return TRUE;
-    p = mat_row_incr (m, p);
-  }
-  return FALSE;
-}
-
-static double
-matrix_row_dot (Matrix * m, gint r1, gint r2)
-{
-  double *p = mat_ptr (m, r1, 0);
-  double *q = mat_ptr (m, r2, 0);
-  double sum = 0;
-  gint i;
-  for (i = 0; i < matrix_cols (m); ++i) {
-    sum += (*p) * (*q);
-    p = mat_col_incr (m, p);
-    q = mat_col_incr (m, q);
-  }
-  return sum;
-}
-
-static void
-matrix_subtract_scaled_row_from_row (Matrix * m, double scale, gint r,
-				     gint r_sub_from)
-{
-  double *p = mat_ptr (m, r_sub_from, 0);
-  double *q = mat_ptr (m, r, 0);
-  gint i;
-  if (fabs (scale) < EPSILON)
-    return;
-  for (i = 0; i < matrix_cols (m); ++i) {
-    *p -= scale * (*q);
-    p = mat_col_incr (m, p);
-    q = mat_col_incr (m, q);
-  }
-}
-
-#ifdef EXTRA_UNUSED_FN
-static void
-matrix_apply (Matrix * m, const double *vec, double *prod)
-{
-  gint i, j;
-
-  for (j = 0; j < matrix_rows (m); ++j)
-    for (i = 0; i < matrix_cols (m); ++i)
-      prod[j] += mat (m, j, i) * vec[i];
-}
-#endif
-
-static gint *
-LU_decompose_in_place (Matrix * m)
-{
-  gint *perm;
-  gint N, pi, i, j, k, max_ind;
-  double x, max;
+  struct CustomLUInfo info;
+  double x, y, w, h;
 
   g_return_val_if_fail (m != NULL, NULL);
-  g_return_val_if_fail (matrix_rows (m) == matrix_cols (m), NULL);
-
-  N = matrix_rows (m);
-  perm = guppi_new0 (gint, N);
-  pi = 0;
-
-  for (k = 0; k < N - 1; ++k) {
-
-    /* Find largest element */
-    max = fabs (mat (m, k, k));
-    max_ind = k;
-    for (i = k + 1; i < N; ++i) {
-      x = fabs (mat (m, i, k));
-      if (max < x) {
-	max_ind = i;
-	max = x;
-      }
-    }
-    perm[pi] = max_ind;
-    ++pi;
-
-    /* Swap */
-    for (i = k; i < N; ++i) {
-      x = mat (m, k, i);
-      mat (m, k, i) = mat (m, max_ind, i);
-      mat (m, max_ind, i) = x;
-    }
-
-    if (fabs (x = mat (m, k, k)) > EPSILON) {
-      for (i = k + 1; i < N; ++i) {
-	mat (m, i, k) /= x;
-      }
-
-      for (i = k + 1; i < N; ++i) {
-	x = mat (m, i, k);
-	for (j = k + 1; j < N; ++j) {
-	  mat (m, i, j) -= x * mat (m, k, j);
-	}
-      }
-    }
-  }
-
-  return perm;
-}
-
-static double *
-LU_solve (Matrix * LU, gint * perm, double *vec,
-	  double x0, double x1, double y0, double y1)
-{
-  gint i, j, k, N;
-  double x, y, w, h;
-  double *soln;
+  g_return_val_if_fail (vec != NULL, NULL);
 
+  g_return_val_if_fail (guppi_matrix_is_square (m), NULL);
+  
   guppi_2sort (&x0, &x1);
   guppi_2sort (&y0, &y1);
 
@@ -263,69 +74,15 @@ LU_solve (Matrix * LU, gint * perm, double *vec,
   x = (x0 + x1) / 2;
   y = (y0 + y1) / 2;
 
-  x0 = x - w / 3;
-  x1 = x + w / 3;
-  y0 = y - h / 3;
-  y1 = y + h / 3;
+  info.x0 = x - w / 3;
+  info.x1 = x + w / 3;
+  info.y0 = y - h / 3;
+  info.y1 = y + h / 3;
 
-  g_return_val_if_fail (LU != NULL, NULL);
-  g_return_val_if_fail (perm != NULL, NULL);
-  g_return_val_if_fail (vec != NULL, NULL);
-
-  g_return_val_if_fail (matrix_cols (LU) == matrix_rows (LU), NULL);
-
-  N = matrix_cols (LU);
-  soln = guppi_new0 (double, N);
-  for (i = 0; i < N; ++i)
-    soln[i] = vec[i];
-
-  /* Apply gauss transforms and permutations */
-  for (k = 0; k < N - 1; ++k) {
-    i = perm[k];
-    x = soln[k];
-    soln[k] = soln[i];
-    soln[i] = x;
-
-    for (i = k + 1; i < N; ++i)
-      soln[i] -= soln[k] * mat (LU, i, k);
-  }
-
-  /* Do back-substitution */
-  if (fabs (mat (LU, N - 1, N - 1)) > EPSILON)
-    soln[N - 1] /= mat (LU, N - 1, N - 1);
-  else {
-    if ((N - 1) % 4 == 0)
-      soln[N - 1] = x0;
-    else if ((N - 1) % 4 == 1)
-      soln[N - 1] = x1;
-    else if ((N - 1) % 4 == 2)
-      soln[N - 1] = y1;
-    else
-      soln[N - 1] = y0;
-  }
-
-  for (i = N - 2; i >= 0; --i) {
-    x = soln[i];
-    for (j = i + 1; j < N; ++j)
-      x -= mat (LU, i, j) * soln[j];
-    if (fabs (mat (LU, i, i)) > EPSILON)
-      soln[i] = x / mat (LU, i, i);
-    else {
-      if (i % 4 == 0)
-	soln[i] = x0;
-      else if (i % 4 == 1)
-	soln[i] = x1;
-      else if (i % 4 == 2)
-	soln[i] = y1;
-      else
-	soln[i] = y0;
-    }
-  }
-
-  return soln;
+  return guppi_matrix_solve_with_fallback (m, vec, custom_solve_fallback, &info);
 }
-
-/****************************************************************************/
+ 
+/**************************************************************************/
 
 enum {
   LEFT, RIGHT, TOP, BOTTOM, WIDTH, HEIGHT,
@@ -390,7 +147,7 @@ term_free (Term * t)
   }
 }
 
-/****************************************************************************/
+/**************************************************************************/
 
 typedef struct _Rule Rule;
 
@@ -509,7 +266,7 @@ rule_list_free (GList * l)
   g_list_free (l);
 }
 
-/****************************************************************************/
+/**************************************************************************/
 
 typedef struct _GuppiLayoutPrivate GuppiLayoutPrivate;
 struct _GuppiLayoutPrivate {
@@ -518,9 +275,8 @@ struct _GuppiLayoutPrivate {
 
   gint N_geom, N_rules;
 
-  Matrix *rule_matrix;
-  Matrix *LU_coor_matrix;
-  gint *LU_perm;
+  GuppiMatrix *rule_matrix;
+  GuppiMatrix *coor_matrix;
 
   gboolean calculating_layout;
   double x0, y0, x1, y1;
@@ -539,14 +295,11 @@ reset_precalculated_layout (GuppiLayout * lay)
 
   p = priv (lay);
 
-  matrix_free (p->rule_matrix);
+  guppi_matrix_free (p->rule_matrix);
   p->rule_matrix = NULL;
 
-  matrix_free (p->LU_coor_matrix);
-  p->LU_coor_matrix = NULL;
-
-  guppi_free (p->LU_perm);
-  p->LU_perm = NULL;
+  guppi_matrix_free (p->coor_matrix);
+  p->coor_matrix = NULL;
 }
 
 static GList *
@@ -836,12 +589,12 @@ remove_rule (GuppiLayout * lay, Rule * r)
 }
 #endif
 
-static Matrix *
+static GuppiMatrix *
 build_rule_matrix (GuppiLayout * lay)
 {
   GuppiLayoutPrivate *p;
   gint R, M, W;
-  Matrix *m;
+  GuppiMatrix *m;
   GList *ri;
   gint r, c;
 
@@ -856,7 +609,7 @@ build_rule_matrix (GuppiLayout * lay)
    * screen bounds and CONSTANT
    */
   W = 6 * M + 5;
-  m = matrix_new (R, W);
+  m = guppi_matrix_new (R, W);
   r = 0;
   for (ri = p->rules; ri != NULL; ri = g_list_next (ri)) {
     Rule *rule = (Rule *) ri->data;
@@ -876,7 +629,7 @@ build_rule_matrix (GuppiLayout * lay)
       else
 	c = code_ext (lay, term);
 
-      mat (m, r, c) += term->c;
+      guppi_matrix_entry (m, r, c) += term->c;
     }
     ++r;
   }
@@ -890,40 +643,40 @@ build_rule_matrix (GuppiLayout * lay)
  */
 
 
-static Matrix *
-simplify_rule_matrix (Matrix * m)
+static GuppiMatrix *
+simplify_rule_matrix (GuppiMatrix *m)
 {
   gint r, j;
   gint nonzero_rows = 0;
   g_return_val_if_fail (m != NULL, NULL);
 
-  for (r = 0; r < matrix_rows (m); ++r) {
+  for (r = 0; r < guppi_matrix_rows (m); ++r) {
     for (j = 0; j < r; ++j) {
-      double sc = matrix_row_dot (m, r, j);
-      matrix_subtract_scaled_row_from_row (m, sc, j, r);
+      double sc = guppi_matrix_row_dot (m, r, j);
+      guppi_matrix_subtract_scaled_row_from_row (m, sc, j, r);
     }
-    if (matrix_row_is_nonzero (m, r)) {
-      matrix_normalize_row (m, r);
+    if (guppi_matrix_row_is_nonzero (m, r)) {
+      guppi_matrix_normalize_row (m, r);
       ++nonzero_rows;
     }
   }
 
-  if (nonzero_rows != matrix_rows (m)) {
-    Matrix *collapsed = matrix_new (nonzero_rows, matrix_cols (m));
+  if (nonzero_rows != guppi_matrix_rows (m)) {
+    GuppiMatrix *collapsed = guppi_matrix_new (nonzero_rows, guppi_matrix_cols (m));
     gint i = 0;
 
     if (guppi_is_very_verbose ())
-      g_message ("redundant rules: %d", matrix_rows (m) - nonzero_rows);
+      g_message ("redundant rules: %d", guppi_matrix_rows (m) - nonzero_rows);
 
-    for (r = 0; r < matrix_rows (m); ++r) {
-      if (matrix_row_is_nonzero (m, r)) {
-	for (j = 0; j < matrix_cols (m); ++j)
-	  mat (collapsed, i, j) = mat (m, r, j);
+    for (r = 0; r < guppi_matrix_rows (m); ++r) {
+      if (guppi_matrix_row_is_nonzero (m, r)) {
+	for (j = 0; j < guppi_matrix_cols (m); ++j)
+	  guppi_matrix_entry (collapsed, i, j) = guppi_matrix_entry (m, r, j);
 	++i;
       }
     }
     g_assert (nonzero_rows == i);
-    matrix_free (m);
+    guppi_matrix_free (m);
     m = collapsed;
   }
 
@@ -932,30 +685,30 @@ simplify_rule_matrix (Matrix * m)
 
 /* Build our matrix of unknowns coordinates */
 
-static Matrix *
-build_coordinate_matrix (GuppiLayout * lay, Matrix * rule_matrix)
+static GuppiMatrix *
+build_coordinate_matrix (GuppiLayout *lay, GuppiMatrix *rule_matrix)
 {
   GuppiLayoutPrivate *p;
-  Matrix *m;
+  GuppiMatrix *m;
   gint C, R;
   gint i, j;
 
   p = priv (lay);
 
   C = 4 * p->N_geom;
-  R = matrix_rows (rule_matrix);
+  R = guppi_matrix_rows (rule_matrix);
   if (R < C)
     g_warning ("Under-specified layout! (rules=%d, coordinates=%d)", R, C);
   if (R > C)
     g_warning ("Over-specified layout! (rules=%d, coordinates=%d)", R, C);
 
-  m = matrix_new (C, C);
+  m = guppi_matrix_new (C, C);
 
   /* Copy the piece of the rule matrix related to coordinates
      over to the coordinate matrix. */
   for (i = 0; i < R; ++i)
     for (j = 0; j < C; ++j)
-      mat (m, i, j) = mat (rule_matrix, i, j);
+      guppi_matrix_entry (m, i, j) = guppi_matrix_entry (rule_matrix, i, j);
 
   /*
    * If our system is under-specified, the last C-R rows of our matrix
@@ -967,7 +720,7 @@ build_coordinate_matrix (GuppiLayout * lay, Matrix * rule_matrix)
 }
 
 static void
-coordinate_matrix_diagnostics (GuppiLayout * lay, Matrix * coor_matrix)
+coordinate_matrix_diagnostics (GuppiLayout *lay, GuppiMatrix *coor_matrix)
 {
   GuppiLayoutPrivate *p;
   GList *gi;
@@ -980,10 +733,10 @@ coordinate_matrix_diagnostics (GuppiLayout * lay, Matrix * coor_matrix)
   for (gi = p->geometries; gi != NULL; gi = g_list_next (gi)) {
     gboolean l, r, t, b;
 
-    l = !matrix_column_is_nonzero (coor_matrix, i++);
-    r = !matrix_column_is_nonzero (coor_matrix, i++);
-    t = !matrix_column_is_nonzero (coor_matrix, i++);
-    b = !matrix_column_is_nonzero (coor_matrix, i++);
+    l = ! guppi_matrix_column_is_nonzero (coor_matrix, i++);
+    r = ! guppi_matrix_column_is_nonzero (coor_matrix, i++);
+    t = ! guppi_matrix_column_is_nonzero (coor_matrix, i++);
+    b = ! guppi_matrix_column_is_nonzero (coor_matrix, i++);
 
     if (l || r || t || b) {
       GuppiGeometry *g = (GuppiGeometry *) gi->data;
@@ -1003,59 +756,18 @@ coordinate_matrix_diagnostics (GuppiLayout * lay, Matrix * coor_matrix)
   }
 }
 
-static void
-LU_coordinate_matrix_diagnostics (GuppiLayout * lay, Matrix * LU_coor_matrix)
-{
-  GuppiLayoutPrivate *p;
-  GList *gi;
-  gint i;
-
-  p = priv (lay);
-  gi = p->geometries;
-
-  i = 0;
-  for (gi = p->geometries; gi != NULL; gi = g_list_next (gi)) {
-    gboolean l, r, t, b;
-
-    l = fabs (mat (LU_coor_matrix, i, i)) < EPSILON;
-    ++i;
-    r = fabs (mat (LU_coor_matrix, i, i)) < EPSILON;
-    ++i;
-    t = fabs (mat (LU_coor_matrix, i, i)) < EPSILON;
-    ++i;
-    b = fabs (mat (LU_coor_matrix, i, i)) < EPSILON;
-    ++i;
-
-    if (l || r || t || b) {
-      GuppiGeometry *g = (GuppiGeometry *) gi->data;
-      g_print ("Zero pivots for %s: ",
-	       gtk_type_name (GTK_OBJECT_TYPE
-			      (guppi_geometry_user_data (g))));
-      if (l)
-	g_print ("left ");
-      if (r)
-	g_print ("right ");
-      if (t)
-	g_print ("top ");
-      if (b)
-	g_print ("bottom ");
-      g_print ("\n");
-    }
-  }
-}
-
-static double *
-build_coordinate_vec (GuppiLayout * lay,
-		      Matrix * rule_matrix,
+static GuppiVector *
+build_coordinate_vec (GuppiLayout *lay,
+		      GuppiMatrix *rule_matrix,
 		      double x0, double y0, double x1, double y1)
 {
   GuppiLayoutPrivate *p;
-  double *vec = guppi_new0 (double, matrix_rows (rule_matrix));
+  GuppiVector *vec = guppi_vector_new (guppi_matrix_rows (rule_matrix));
   gint i;
 
   p = priv (lay);
 
-  for (i = 0; i < matrix_rows (rule_matrix); ++i) {
+  for (i = 0; i < guppi_matrix_rows (rule_matrix); ++i) {
     GList *gi;
     double c;
     gint j;
@@ -1067,58 +779,58 @@ build_coordinate_vec (GuppiLayout * lay,
       double c;
       double sz;
 
-      c = mat (rule_matrix, i, j);
-      if (fabs (c) > EPSILON) {
+      c = guppi_matrix_entry (rule_matrix, i, j);
+      if (fabs (c) > rule_matrix->epsilon) {
 	sz = guppi_geometry_natural_width (geom);
-	vec[i] -= c * MAX (sz, 0);
+	guppi_vector_entry (vec, i) -= c * MAX (sz, 0);
       }
       ++j;
 
-      c = mat (rule_matrix, i, j);
-      if (fabs (c) > EPSILON) {
+      c = guppi_matrix_entry (rule_matrix, i, j);
+      if (fabs (c) > rule_matrix->epsilon) {
 	sz = guppi_geometry_natural_height (geom);
-	vec[i] -= c * MAX (sz, 0);
+	guppi_vector_entry (vec, i) -= c * MAX (sz, 0);
       }
       ++j;
     }
 
-    c = mat (rule_matrix, i, j);
-    if (fabs (c) > EPSILON) {
-      vec[i] -= c * x0;
+    c = guppi_matrix_entry (rule_matrix, i, j);
+    if (fabs (c) > rule_matrix->epsilon) {
+      guppi_vector_entry (vec, i) -= c * x0;
     }
     ++j;
 
-    c = mat (rule_matrix, i, j);
-    if (fabs (c) > EPSILON) {
-      vec[i] -= c * x1;
+    c = guppi_matrix_entry (rule_matrix, i, j);
+    if (fabs (c) > rule_matrix->epsilon) {
+      guppi_vector_entry (vec, i) -= c * x1;
     }
     ++j;
 
-    c = mat (rule_matrix, i, j);
-    if (fabs (c) > EPSILON) {
-      vec[i] -= c * y0;
+    c = guppi_matrix_entry (rule_matrix, i, j);
+    if (fabs (c) > rule_matrix->epsilon) {
+      guppi_vector_entry (vec, i) -= c * y0;
     }
     ++j;
 
-    c = mat (rule_matrix, i, j);
-    if (fabs (c) > EPSILON) {
-      vec[i] -= c * y1;
+    c = guppi_matrix_entry (rule_matrix, i, j);
+    if (fabs (c) > rule_matrix->epsilon) {
+      guppi_vector_entry (vec, i) -= c * y1;
     }
     ++j;
 
-    c = mat (rule_matrix, i, j);	/* constant term */
-    if (fabs (c) > EPSILON) {
-      vec[i] -= c;
+    c = guppi_matrix_entry (rule_matrix, i, j);	/* constant term */
+    if (fabs (c) > rule_matrix->epsilon) {
+      guppi_vector_entry (vec, i) -= c;
     }
     ++j;
 
-    g_assert (j == matrix_cols (rule_matrix));
+    g_assert (j == guppi_matrix_cols (rule_matrix));
   }
 
   return vec;
 }
 
-/*****************************************************************************/
+/***************************************************************************/
 
 void
 guppi_layout_natural_width (GuppiLayout * lay, GuppiGeometry * geom)
@@ -1778,7 +1490,7 @@ guppi_layout_same_y_center (GuppiLayout * lay,
   add_rule (lay, r);
 }
 
-/*****************************************************************************/
+/***************************************************************************/
 
 void
 guppi_layout_remove (GuppiLayout * lay, GuppiGeometry * geom)
@@ -1858,7 +1570,7 @@ guppi_layout_replace (GuppiLayout * lay,
 }
 
 
-/*****************************************************************************/
+/***************************************************************************/
 
 void
 guppi_layout_bounds (GuppiLayout * lay,
@@ -1917,8 +1629,8 @@ void
 guppi_layout_calc (GuppiLayout * lay)
 {
   GuppiLayoutPrivate *p;
-  double *v;
-  double *s;
+  GuppiVector *v;
+  GuppiVector *s;
   gint i = 0;
   GList *iter;
 
@@ -1940,35 +1652,31 @@ guppi_layout_calc (GuppiLayout * lay)
   p->calculating_layout = TRUE;
 
   if (p->rule_matrix == NULL) {
-    Matrix *rule_matrix;
-    Matrix *coor_matrix;
-    gint *perm;
+    GuppiMatrix *rule_matrix;
+    GuppiMatrix *coor_matrix;
 
     rule_matrix = build_rule_matrix (lay);
     rule_matrix = simplify_rule_matrix (rule_matrix);
     coor_matrix = build_coordinate_matrix (lay, rule_matrix);
     coordinate_matrix_diagnostics (lay, coor_matrix);
-    perm = LU_decompose_in_place (coor_matrix);
-    LU_coordinate_matrix_diagnostics (lay, coor_matrix);
 
     p->rule_matrix = rule_matrix;
-    p->LU_coor_matrix = coor_matrix;
-    p->LU_perm = perm;
+    p->coor_matrix = coor_matrix;
   }
 
   v = build_coordinate_vec (lay, p->rule_matrix, p->x0, p->y0, p->x1, p->y1);
-  s = LU_solve (p->LU_coor_matrix, p->LU_perm, v, p->x0, p->y0, p->x1, p->y1);
-
+  s = custom_solve (p->coor_matrix, v, p->x0, p->y0, p->x1, p->y1);
+  
   if (s != NULL) {
     iter = p->geometries;
     while (iter != NULL) {
       GuppiGeometry *geom = (GuppiGeometry *) iter->data;
 
       guppi_geometry_set_position (geom,
-				   CLAMP (s[i], p->x0, p->x1),
-				   CLAMP (s[i + 1], p->x0, p->x1),
-				   CLAMP (s[i + 2], p->y0, p->y1),
-				   CLAMP (s[i + 3], p->y0, p->y1));
+				   CLAMP (guppi_vector_entry (s, i),   p->x0, p->x1),
+				   CLAMP (guppi_vector_entry (s, i+1), p->x0, p->x1),
+				   CLAMP (guppi_vector_entry (s, i+2), p->y0, p->y1),
+				   CLAMP (guppi_vector_entry (s, i+3), p->y0, p->y1));
 
 #if 0
       g_print ("%s: L:%g R:%g T:%g B:%g\n",
@@ -1984,8 +1692,8 @@ guppi_layout_calc (GuppiLayout * lay)
     }
   }
 
-  guppi_free (v);
-  guppi_free (s);
+  guppi_vector_free (v);
+  guppi_vector_free (s);
 
   p->calculating_layout = FALSE;
 }
@@ -2025,4 +1733,4 @@ guppi_layout_calc_flush (GuppiLayout * lay)
 
 }
 
-/* $Id: guppi-layout.c,v 1.22 2001/01/16 23:36:26 trow Exp $ */
+/* $Id: guppi-layout.c,v 1.24 2001/05/06 08:26:43 trow Exp $ */
