@@ -1,4 +1,4 @@
-/* $Id: guppi-splash.h,v 1.9 2000/10/27 18:23:05 trow Exp $ */
+/* $Id: guppi-splash.h,v 1.10 2001/01/16 23:36:40 trow Exp $ */
 
 /*
  * guppi-splash.h
@@ -27,7 +27,7 @@
 #ifndef _INC_GUPPI_SPLASH_H
 #define _INC_GUPPI_SPLASH_H
 
-#include <gtk/gtk.h>
+/* #include <gtk/gtk.h> */
 
 #include <guppi-defs.h>
 
@@ -42,4 +42,4 @@ END_GUPPI_DECLS
 
 #endif /* _INC_GUPPI_SPLASH_H */
 
-/* $Id: guppi-splash.h,v 1.9 2000/10/27 18:23:05 trow Exp $ */
+/* $Id: guppi-splash.h,v 1.10 2001/01/16 23:36:40 trow Exp $ */
